

// The scale equation calculated by Vernier's Graphical Analysis
float getScatterScale(float fCos)
{
	float x = 1.0 - fCos;
	return 0.25 * exp(-0.00287 + x*(0.459 + x*(3.83 + x*(-6.80 + x*5.25))));
}

// Calculates the Mie phase function
float getMiePhase(float fCos, float fCos2)
{
	float3 fmie = float3( 1.5 * (1-c_mieG*c_mieG) / (2 + c_mieG*c_mieG), 1 + c_mieG*c_mieG, -2.0f * c_mieG );
	return fmie.x * (1 + fCos2) * pow(fmie.y + fmie.z * fCos, -1.5);
}

// Calculates the Rayleigh phase function
float getRayleighPhase(float fCos2)
{
	//return 1.0;
	return 0.75 + 0.75*fCos2;
}

// final scattering is c0 + obj_color * c1
void GetAtmScatter(	in float3 v3Pos, in float3 v3Norm, 
					in float4 v3Ray,		// xyz - normalized dir from cam to pos, w - dist to cam
					out float3 col0, out float3 col1)
{
	const float2 c0 = float3(0,0.5,1);
	const float fKrESun = c_Kray * c_SunIntensity;
	const float fKmESun = c_Kmie * c_SunIntensity;
	const float fKr4PI = c_Kray * 4.0f * 3.14159268f;
	const float fKm4PI = c_Kmie * 4.0f * 3.14159268f;
	const float fScaleOverScaleDepth = c_iSkyHeight / c_ScaleDepth;
	const float fDepth = exp( -dot(c_GroundPlane, c_CameraPos) * fScaleOverScaleDepth);

	// Calculate the ray's starting position, then calculate its scattering offset
	float4 v3Start = c_InnerRadius.yxyy * c_GroundPlane.xyzw + c_CameraPos;
	float fCameraAngle = dot(-v3Ray.xyz, v3Pos);
	float fLightAngle = dot(c_SunDir, v3Norm);
	float fCameraScale = getScatterScale(fCameraAngle);
	float fLightScale = getScatterScale(fLightAngle);
	float fCameraOffset = fDepth*fCameraScale;
	float fTemp = (fLightScale + fCameraScale);

	// Initialize the scattering loop variables
	//gl_FrontColor = vec4(0.0, 0.0, 0.0, 0.0);
	float fSampleLength = v3Ray.w * fiSamples;
	float fScaledLength = fSampleLength * c_iSkyHeight;

	float3 v3SampleRay = v3Ray.xyz * fSampleLength;

	// w will always be 1 so that can get height easier
	float fHeight = dot(v3Start + v3SampleRay.xyzz * c0.yyyx, c_GroundPlane);
	float fDHeight = dot(v3SampleRay.xyz, c_GroundPlane.xyz);

	// Now loop through the sample rays
	float3 v3FrontColor = float3(0.0, 0.0, 0.0);
	float3 v3Attenuate;
	for(int i=0; i<c_Samples; i++)
	{
		float fDepth = exp(-fScaleOverScaleDepth * fHeight);
		float fScatter = fDepth*fTemp - fCameraOffset;
		v3Attenuate = exp(-fScatter * (c_InvWavelength * fKr4PI + fKm4PI));
		v3FrontColor += v3Attenuate * (fDepth * fScaledLength);
		fHeight += fDHeight;
	}

	col0 = v3FrontColor * (c_InvWavelength * fKrESun + fKmESun);
	col1 = v3Attenuate;
}

// Returns the near intersection point of a line and a sphere
float getNearIntersection(float3 v3Pos, float3 v3Ray, float fDistance2, float fRadius2)
{
	float B = 2.0 * dot(v3Pos, v3Ray);
	float C = fDistance2 - fRadius2;
	float fDet = max(0.0, B*B - 4.0 * C);
	return 0.5 * (-B - sqrt(fDet));
}

// Returns the far intersection point of a line and a sphere
float getFarIntersection(float3 v3Pos, float3 v3Ray, float fDistance2, float fRadius2)
{
	float B = 2.0 * dot(v3Pos, v3Ray);
	float C = fDistance2 - fRadius2;
	float fDet = max(0.0, B*B - 4.0 * C);
	return 0.5 * (-B + sqrt(fDet));
}
