// beginning of luminance computation on a 64x64 texture
texture g_txSrc < int postrt = 7; >;

sampler2D g_sSrc = sampler_state {
    Texture = <g_txSrc>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
};

static const int SAMPLES = 8;

const float2 PixelKernel1[SAMPLES] = {
	1,1,
	2,2,
	3,3,
	4,4,
	5,5,
	6,6,
	7,7,
	8,8 };

const float2 PixelKernel2[SAMPLES] = {
	16,16,
	24,24,
	32,32,
	40,40,
	48,48,
	56,56,
	64,64,
	72,72
	};


float2 TexelKernel1[SAMPLES] < string ConvertPixelsToTexels = "PixelKernel1"; >;
float2 TexelKernel2[SAMPLES] < string ConvertPixelsToTexels = "PixelKernel2"; >;

const float4 TexelWeights1[SAMPLES] = {
	0.5 * pow(0.97, 1) * float4(0.5f, 0.5f, 0.5f,  0.0f),	// w
	0.5 * pow(0.97, 2) * float4(0.8f, 0.3f, 0.3f,  0.0f),
	0.5 * pow(0.97, 3) * float4(1.0f, 0.2f, 0.2f,  0.0f),	// r
	0.5 * pow(0.97, 4) * float4(0.5f, 0.2f, 0.6f,  0.0f),
	0.5 * pow(0.97, 5) * float4(0.2f, 0.2f, 1.0f,  0.0f),	// b
	0.5 * pow(0.97, 6) * float4(0.2f, 0.3f, 0.7f,  0.0f),
	0.5 * pow(0.97, 7) * float4(0.2f, 0.6f, 0.2f,  0.0f),	// g
	0.5 * pow(0.97, 8) * float4(0.3f, 0.5f, 0.3f,  0.0f) };

const float4 TexelWeights2[SAMPLES] = {
	1.5 * pow(0.97, 16) * float4(0.5f, 0.5f, 0.5f,  0.0f),	// w
	1.5 * pow(0.97, 24) * float4(0.8f, 0.3f, 0.3f,  0.0f),
	1.5 * pow(0.97, 32) * float4(1.0f, 0.2f, 0.2f,  0.0f),	// r
	1.5 * pow(0.97, 40) * float4(0.5f, 0.2f, 0.6f,  0.0f),
	1.5 * pow(0.97, 48) * float4(0.2f, 0.2f, 1.0f,  0.0f),	// b
	1.5 * pow(0.97, 56) * float4(0.2f, 0.3f, 0.7f,  0.0f),
	1.5 * pow(0.97, 64) * float4(0.2f, 0.6f, 0.2f,  0.0f),	// g
	1.5 * pow(0.97, 72) * float4(0.3f, 0.5f, 0.3f,  0.0f)
	};

float4 StarEffect(uniform float2 TexelKernel[SAMPLES], const uniform float4 TexelWeights[SAMPLES],
				const uniform float2 fOffMult,
					in float2 vScreenPosition : TEXCOORD0) : COLOR
{
    float4 vColor = float4(0,0,0, 1);

    for(int iSample = 0; iSample < 6; iSample++) {
        vColor.xyz += TexelWeights[iSample] * tex2D(g_sSrc, vScreenPosition + fOffMult*TexelKernel[iSample]).xyz;
    }
    return vColor*0.25;
}

technique tech0 <	int postrt = 8;
					float scalex = 0.25f;
					float scaley = 0.25f;
					string usepost0 = "PP_BloomV";
				> {
	pass p00 {
		PixelShader = compile ps_2_0 StarEffect(TexelKernel1, TexelWeights1, 1*float2(1,1));
	}
	/*pass p01 {
		AlphaBlendEnable = True;
		SrcBlend = One;	DestBlend = One;
		PixelShader = compile ps_2_0 StarEffect(TexelKernel2, TexelWeights2, float2(1,1));
	}*/
	pass p10 {
		AlphaBlendEnable = True;
		SrcBlend = One;	DestBlend = One;
		PixelShader = compile ps_2_0 StarEffect(TexelKernel1, TexelWeights1, 1*float2(1,-1));
	}
	/*pass p11 {
		AlphaBlendEnable = True;
		SrcBlend = One;	DestBlend = One;
		PixelShader = compile ps_2_0 StarEffect(TexelKernel2, TexelWeights2, float2(1,-1));
	}*/
	pass p21 {
		AlphaBlendEnable = True;
		SrcBlend = One;	DestBlend = One;
		PixelShader = compile ps_2_0 StarEffect(TexelKernel1, TexelWeights1, 1*float2(-1,1));
	}
	/*pass p21 {
		AlphaBlendEnable = True;
		SrcBlend = One;	DestBlend = One;
		PixelShader = compile ps_2_0 StarEffect(TexelKernel2, TexelWeights2, float2(-1,1));
	}*/
	pass p31 {
		AlphaBlendEnable = True;
		SrcBlend = One;	DestBlend = One;
		PixelShader = compile ps_2_0 StarEffect(TexelKernel1, TexelWeights1, 1*float2(-1,-1));
	}
	/*pass p32 {
		AlphaBlendEnable = True;
		SrcBlend = One;	DestBlend = One;
		PixelShader = compile ps_2_0 StarEffect(TexelKernel2, TexelWeights2, float2(-1,-1));
	}*/
}
