texture texParticle;

shared float c_ElapsedTime;

float fPartSize;
float2 g_fiNumDivs = float2(0.5, 0.5);
float4x4 m_WVP;		// world view proj matrix

sampler2D g_texture =
sampler_state
{
    Texture = <texParticle>;
    AddressU = Wrap;
    AddressV = Wrap;
    MinFilter = Point;
    MagFilter = Linear;
    MipFilter = Linear;
};

// Regular shader
struct VSINPUT
{
	float4 vpos : POSITION0;
	half4 color : COLOR0;
	half2 tex : TEXCOORD0;
};

struct PSINPUT
{
	float4 vpos : POSITION0;
	half4 color : COLOR0;
	half2 tex : TEXCOORD0;
};

PSINPUT RegVS(VSINPUT i) {
	PSINPUT o;
	
	o.vpos = mul(i.vpos, m_WVP);
	o.tex = i.tex;
	o.color = i.color;
	return o;
}

half4 RegPS(PSINPUT i) : COLOR
{
	return i.color * tex2D(g_texture, i.tex).xyzz;
}

// Point sprites shader
struct POINT_VSINPUT
{
	float4 vpos : POSITION0;
	half4 color : COLOR0;
	float2 info : TEXCOORD0; //.x - size, .y - tex coord start
};

struct POINT_PSINPUT
{
	float4 vpos : POSITION0;
	float radius : PSIZE;
	half4 color : COLOR0;
	half4 texindex : COLOR1;
	float2 tex : TEXCOORD0;
};

POINT_PSINPUT PointVS(POINT_VSINPUT i) {
	POINT_PSINPUT o;
	
	o.vpos = mul(i.vpos, m_WVP);
	o.radius = fPartSize * i.info.x * 256.0f / o.vpos.w;
	o.color = i.color;
	o.tex.xy = 0;
	o.texindex.x = i.info.y;
	o.texindex.yzw = 0;
	return o;
}

float4 PointPS(POINT_PSINPUT i) : COLOR
{
	half2 newtex = i.tex * g_fiNumDivs + i.texindex;
	float4 c = tex2D(g_texture, newtex);
	return i.color * c.xxxx;
}

technique regtech {
	pass P0 {
		AlphaTestEnable = True;
		AlphaFunc = GreaterEqual;
		AlphaRef = 20;
		VertexShader = compile vs_1_1 RegVS();
		PixelShader = compile ps_1_1 RegPS();
	}
}

technique pointtech {
	pass P0 {
		AlphaTestEnable = True;
		AlphaFunc = GreaterEqual;
		AlphaRef = 20;
		
		VertexShader = compile vs_1_1 PointVS();
		PixelShader = compile ps_2_0 PointPS();
	}
}
