float4x4 c_WVP;
float3 c_CameraPos;
float4 c_SunDir;
float c_NightTime;
texture t_c_tex;
sampler2D c_tex = sampler_state {
Texture = <t_c_tex>;
MipFilter = LINEAR;
MinFilter = LINEAR;
MagFilter = LINEAR;
};

texture t_c_tHeight;
sampler1D c_tHeight = sampler_state {
Texture = <t_c_tHeight>;
MipFilter = LINEAR;
MinFilter = LINEAR;
MagFilter = LINEAR;
AddressU = Mirror;
};

float c_fCloudTime;
float2 c_fCloudHeight;
float2 c_firadius;
float2 c_itexdims;
float c_size;
float c_alpha;
float4 c_diffuse;
float3 c_amb;
texture t_c_tNight;
sampler1D c_tNight = sampler_state {
Texture = <t_c_tNight>;
MipFilter = LINEAR;
MinFilter = LINEAR;
MagFilter = LINEAR;
AddressU = Wrap;
AddressV = Wrap;
};

float c_mult;
// mtrl type: Clouds;
struct VSINPUT {
float4 v_pos : POSITION0;
half4 v_normal : NORMAL0;
half4 v_tex0 : TEXCOORD0;
half4 v_tex1 : TEXCOORD1;
};

struct PSOUTPUT {
	half4 color0 : COLOR0;
	half4 color1 : COLOR1;

};

struct PASS0_PSINPUT
{
float4 v_pos : POSITION0;
float fheight:TEXCOORD0;
float4 dif:COLOR0;
float2 ftex:TEXCOORD1;
};

PASS0_PSINPUT Pass0VS(VSINPUT i)
{
PASS0_PSINPUT o;
float4 b_objpos = i.v_pos;
half4 b_objnorm = i.v_normal;
half4 b_objtex0 = i.v_tex0;
half4 b_objtex1 = i.v_tex1;
half4 dif;

// inputs: pos, vel, texinfo.xy (tex coords), texinfo.zw (rand)
float rand = b_objtex1.x / 256.0f + b_objtex1.w;

// get real pos (something random)
half3 vup = float3(0,1,0) - b_objnorm.y * b_objnorm.xyz;
half s1 = sin(0.8f * c_fCloudTime * rand);
half s2 = sin(0.1f * c_fCloudTime + rand);

float3 len;
len.x = dot(b_objpos, b_objpos);
len.y = rsqrt(len.x);
len.z = c_firadius.y * len.x * len.y; // radius with respect to cloud radius

half3 norm = b_objpos * len.y;

b_objpos.xyz += 15 * (s1 * rand + (1.5 - rand) * s2) * vup + b_objnorm.xyz * 10 * s1;

float fheight = c_fCloudHeight.x * b_objpos.y + c_fCloudHeight.y;
o.fheight = fheight;


// find real position
dif.a = max(c_alpha * (1.0 - 0.8*len.z) * (0.75 + 0.25 * rand), 0);
float size = c_size * (dif.a + 0.75 * rand);

const float4 c0 = float4(2,-2,-1,1);

half3 fcamdir = normalize(c_CameraPos - b_objpos.xyz);
half3 fup = normalize(float3(0,1,0) - fcamdir.xyz * fcamdir.y);
half3 fright = cross(fup, fcamdir);
half2 fext = size * (c0.xy * b_objtex0.xw + c0.zw);
b_objpos.xyz += fright * fext.x + fup * fext.y;

dif.xyz = c_diffuse.xyz * (dot(c_SunDir, norm)) + c_amb;
o.dif = dif;


float2 ftex = b_objtex0.zy + b_objtex0.xw * c_itexdims;
o.ftex = ftex;



o.v_pos = mul(b_objpos, c_WVP);
return o;
}

PSOUTPUT Pass0PS(PASS0_PSINPUT i)
{
PSOUTPUT o;
float fheight = i.fheight;
float4 dif = i.dif;
float2 ftex = i.ftex;
o.color0 = 0;
o.color1 = 0;
float3 blend = tex2D(c_tex, ftex).rgb;
//o.color0.xyz = dif + tex1D(c_tHeight, fheight);

o.color0.xyz = blend*dif + c_diffuse.a*lerp(tex1D(c_tHeight, fheight), tex1D(c_tNight, fheight), c_NightTime);
o.color0.a = blend*dif.a;
o.color1.rgb = blend*0.1;
o.color1.a = 0.5*o.color0.a;


return o;
}

technique tech0
{
	pass P0
	{
AlphaBlendEnable = True;
ZWriteEnable = False;
SrcBlend = SrcAlpha;
DestBlend = InvSrcAlpha;
CullMode = None;
VertexShader = compile vs_2_0 Pass0VS();
PixelShader = compile ps_2_0 Pass0PS();
	}
}

