#ifdef SHADOWFRAME_EDITOR
typedef vector< EVENTINFO, allocator<EVENTINFO> > EVENTS;			// list for actual game??
typedef vector< EVENTS, allocator<EVENTS> > VECEVENTS;
#endif

#ifdef SHADOWFRAME_GAME
typedef list< EVENTINFO, allocator<EVENTINFO> > EVENTS;			// list for actual game??
typedef vector< EVENTS, allocator<EVENTS> > VECEVENTS;
#endif

/// particle systems and their timings
class LevelSFX : public LevelItem
{
	friend class Level;
	friend class CSFXItemDlg;

public:

	static const UINT ET_InRange;
	static const UINT ET_NumEvents;

	struct EventNames
	{
		EventNames()
		{
			vecNames.resize(ET_NumEvents);
			vecNames[ET_InRange] = _T("in range");
		}

		vector< sfstring > vecNames;
	};

#ifdef SHADOWFRAME_EDITOR
	bool Write(FILE* f);
#endif

	bool Read(FILE *f, LPD3DDEV pd3dDevice);

	LIContactCode Move(float fTimeElapsed);
	LIContactCode MakeContact(SpaceObject& pship);

	bool DeleteWpnOnCollision()							{ return false; }
	bool DisappearOnDeactivation()								{ return true; }

	UINT GetShipColIndex()								{ return ET_InRange; }

	DWORD						GetNumEvents()			{ return ET_NumEvents; }

protected:
	LevelSFX(DWORD dwNewId);
	~LevelSFX();

	vector< sfstring >&				GetEventNameVec() const			{ return evnames.vecNames; }

	static EventNames evnames;
};

/// Tall grass that waves with the wind. Manages an extra vb that creates the 
/// grass vertices and an extra effect especially for the grass. The vb's texture coordinates
/// will be 1 at the top (where the grass is most affected by the wind). The wind will
/// have direction and a linear dependence of its strength as a function of x and z (so that the grass can span longer distances)

/// Note that the grass can still have a normal render obj (can be used for a fuzzy grass bed on the bottom)

// holds an extra effect used for custom geometry
class SFExtraRenderInfo : public SFWorldObj::SFRenderInfo
{
	friend class LevelGrass;
	friend class LevelClouds;

protected:
	bool bExtraChanged;
public:
	SFExtraRenderInfo();

	void ResizeObjEffects(UINT uNewSize);

	void SetChange()			{ bExtraChanged = bChanged = true; }

	ISFEffect* GetExtra()						{ return vecEffects.size() > 0 ? vecEffects.back() : NULL; }
	DWORD GetNumObjEffects()					{ return vecEffects.size()-1; }
};

class LevelGrass : public LevelItem
{
	friend class Level;
	friend class CGrassItemDlg;

public:

	static const UINT ET_InRange;
	static const UINT ET_NumEvents;

	struct EventNames
	{
		EventNames()
		{
			vecNames.resize(ET_NumEvents);
			vecNames[ET_InRange] = _T("in range");
		}

		vector< sfstring > vecNames;
	};

#ifdef SHADOWFRAME_EDITOR
	bool Write(FILE* f);
#endif

	bool Read(FILE *f, LPD3DDEV pd3dDevice);

	LIContactCode Move(float fTimeElapsed);
	LIContactCode MakeContact(SpaceObject& pship);

	bool DeleteWpnOnCollision()							{ return false; }
	bool DisappearOnDeactivation()						{ return true; }

	UINT GetShipColIndex()								{ return ET_InRange; }

	DWORD						GetNumEvents()			{ return ET_NumEvents; }

	
	/// if <= 0 leaves it unchanged
	void SetWorldExtents(float fx, float fz, LPD3DDEV pd3dDevice);
	
	/// number of rows and cols (can be 0)
	HRESULT SetDims(int rows, int cols, LPD3DDEV pd3dDevice);
	/// specifies how many vertical breaks in the vb to put in each row and col
	HRESULT SetDensity(int rowdensity, int coldensity, LPD3DDEV pd3dDevice);
	HRESULT SetRandSeed(DWORD dwRandSeed, LPD3DDEV pd3dDevice);
	HRESULT SetHeightSegs(int iHeightSegs, LPD3DDEV pd3dDevice);

	HRESULT SetGrassMtrl(WORD wGrassMtrl);
	HRESULT RebuildVB(LPD3DDEV pd3dDevice);

	virtual void LoadVideoRes(LPD3DDEV pd3dDevice, BYTE byKitaiRes);
	virtual void ReleaseVideoRes(BYTE byKitaiRes);

	virtual HRESULT Render(LPD3DDEV pd3dDevice, RenderPass passtype = RP_Normal);

	inline WORD GetGrassMtrl()	{ return wGrassMtrl; }

	inline WORD GetRows()		{ return wrows; }
	inline WORD GetCols()		{ return wcols; }
	inline WORD GetRowDensity()	{ return wrowdensity; }
	inline WORD GetColDensity()	{ return wcoldensity; }
	inline WORD GetHeightSegs()	{ return wheightsegs; }
	inline DWORD GetRandSeed()	{ return dwrandseed; }

protected:
	void CreateRenderInfo();

	LevelGrass(DWORD dwNewId);
	~LevelGrass();

	vector< sfstring >&				GetEventNameVec() const			{ return evnames.vecNames; }

	WORD wGrassMtrl;
	WORD wrows, wcols, wrowdensity, wcoldensity, wheightsegs;
	DWORD dwrandseed;

	DWORD dwNumVerts, dwNumFaces;
	LPD3DVB pvbgrass;
	LPD3DIB pibgrass;

	static EventNames evnames;
};

/// body of water that reflects its surroundings.. must be planar
class LevelPond : public LevelItem
{
	friend class Level;
	friend class CPondItemDlg;

public:

	static const UINT ET_InRange;
	static const UINT ET_NumEvents;

	struct EventNames
	{
		EventNames()
		{
			vecNames.resize(ET_NumEvents);
			vecNames[ET_InRange] = _T("in range");
		}

		vector< sfstring > vecNames;
	};

#ifdef SHADOWFRAME_EDITOR
	bool Write(FILE* f);
#endif

	bool Read(FILE *f, LPD3DDEV pd3dDevice);

	LIContactCode Move(float fTimeElapsed);
	LIContactCode MakeContact(SpaceObject& pship);

	bool DeleteWpnOnCollision()							{ return false; }
	bool DisappearOnDeactivation()								{ return true; }

	UINT GetShipColIndex()								{ return ET_InRange; }

	DWORD						GetNumEvents()			{ return ET_NumEvents; }

protected:
	LevelPond(DWORD dwNewId);
	~LevelPond();

	vector< sfstring >&				GetEventNameVec() const			{ return evnames.vecNames; }

	static EventNames evnames;
};

// moving body of water. It will reflect like a pond, but not dynamically (static environment map)
class LevelRiver : public LevelItem
{
	friend class Level;
	friend class CRiverItemDlg;

public:

	static const UINT ET_InRange;
	static const UINT ET_NumEvents;

	struct EventNames
	{
		EventNames()
		{
			vecNames.resize(ET_NumEvents);
			vecNames[ET_InRange] = _T("in range");
		}

		vector< sfstring > vecNames;
	};

#ifdef SHADOWFRAME_EDITOR
	bool Write(FILE* f);
#endif

	bool Read(FILE *f, LPD3DDEV pd3dDevice);

	LIContactCode Move(float fTimeElapsed);
	LIContactCode MakeContact(SpaceObject& pship);

	bool DeleteWpnOnCollision()							{ return false; }
	bool DisappearOnDeactivation()								{ return true; }

	UINT GetShipColIndex()								{ return ET_InRange; }

	DWORD						GetNumEvents()			{ return ET_NumEvents; }

protected:
	LevelRiver(DWORD dwNewId);
	~LevelRiver();

	vector< sfstring >&				GetEventNameVec() const			{ return evnames.vecNames; }

	static EventNames evnames;
};

// Round clouds. The clouds will be represented as rotated static particles. Each particle will be associated
// a normal in order to light it with the sun. There will be two ways to generate the particles: randomly place
// them in the box LevelClouds defines, or read multiple smaller boxes from a .x file and randomly place the particles
// inside those (gives more control). The clouds will use the material from SFRenderObj. All the animation
// will be done in the shader (particles will be given a random seed value at creation time). They will
// be synchronized under a fCloudTime shader variable that represents time elapsed. The cloud itself
// will move as time passes, and will wrap around the world bounding box once it reaches the edge. This will cause
// fCloudTime to be reset. Associated with each vertex will be:
// pos - local pos with respect to cloud center, also used as a normal for lighting,
//		extract where the cloud height is from pos.y by a simple linear fn (height = fCloudHeight.x * pos.y + fCloudHeight.y)
//		length of vpos is used to determine how far particle is from cloud center
// tex coord - low res tex coord from cloud texture - 2 bytes
// velocity - low res velocity vector of particle - 3 bytes
// random value - 2 bytes, add along with tex coord
//
// Extra shader constants that will be written are (need to be declared as global constants)
// fCloudTime - time
// fCloudHeight - used to get height in [0,1] where 0 represents lowest particle
// fCloudAlpha - animated transparency for cloud (goes to 0 when about to wrap)
// fSpriteSize - size of all the sprites, the actual sprite size will depend on alpha, time, and how far sprite is from cloud center
#define SFCLOUD_STRATUS		0x01 // long and flat
#define SFCLOUD_SOLID		0x02
#define SFCLOUD_WHISPY		0x03
#define SFCLOUD_GENFROMFILE	0x80

class LevelClouds : public LevelItem
{
	friend class Level;
	friend class CCloudsItemDlg;

	struct CLOUDVERTEX
	{
		DXVEC3 pos, vel;
		BYTE byu, byv, bydirx, bydiry;
		BYTE byrot, byresv;
		WORD wrand;

		static const D3DVERTEXELEMENT9 Decl[5];
		static LPD3DDECL pdecl;		// created in SFLevel
	};

public:

	static const UINT ET_InRange;
	static const UINT ET_NumEvents;

	struct EventNames
	{
		EventNames()
		{
			vecNames.resize(ET_NumEvents);
			vecNames[ET_InRange] = _T("in range");
		}

		vector< sfstring > vecNames;
	};

#ifdef SHADOWFRAME_EDITOR
	bool Write(FILE* f);

	// change only if cloud is generated from the box
	void SetNewExtents(float fx, float fy, float fz);

	void OnCloudsChanged(LPD3DDEV pd3dDevice);
	void SetGenFromFile(sfstring& strNewFile, LPD3DDEV pd3dDevice);
	void SetGenFromBox(LPD3DDEV pd3dDevice);
#endif

	HRESULT SetCloudsMtrl(WORD wMtrl);
	inline WORD GetCloudsMtrl()		{ return wCloudsMtrl; }

	void CreateRenderInfo();

	bool Read(FILE *f, LPD3DDEV pd3dDevice);

	bool DeleteWpnOnCollision()							{ return false; }
	bool DisappearOnDeactivation()						{ return true; }

	UINT GetShipColIndex()								{ return ET_InRange; }
	DWORD						GetNumEvents()			{ return ET_NumEvents; }

	inline BYTE GetCloudType()		{ return byTypeCloud; }

	virtual LIContactCode Move(float fTimeElapsed);
	virtual LIContactCode MakeContact(SpaceObject& pship) { return CC_Nothing; }

	virtual HRESULT Render(LPD3DDEV pd3dDevice, RenderPass passtype = RP_Normal);

protected:
	LevelClouds(DWORD dwNewId);
	~LevelClouds();

	virtual void Destroy();

	// generates random points in a box with fDensity being the density, new vertices are added to vecParticles
#ifdef SHADOWFRAME_EDITOR
	HRESULT GenerateFromFile(LPD3DDEV pd3dDevice);
	HRESULT GenerateFromGlobalBox(LPD3DDEV pd3dDevice);
	void GenerateFromBox(DXVEC3& vPos, DXVEC3& vExtents, vector<CLOUDVERTEX>& vecParticles);	
#endif

	vector< sfstring >&				GetEventNameVec() const			{ return evnames.vecNames; }

#ifdef SHADOWFRAME_EDITOR
	sfstring strBoxFile;			// used to read the boxes that define the cloud
#endif

	// property vars
	DXVEC3 vCloudPos;	// center of local cloud position
	DXVEC3 vVel;	// velocity of whole cloud

	PS_NUMBER<float> fSpriteSize;	// rand for every sprite
	PS_NUMBER<float> fDensity;		// number of sprites per unit volume (rand for every box)
	float fCullDist;				// min distance two sprites can be
	BYTE byTypeCloud;				// SFCLOUD_ const, depending on type, set tex coords and different particle placement schemes
	BYTE byTexRows, byTexCols;		// rows and columns of textures in big atlas texture
	WORD wCloudsMtrl;

	float fCloudRadius;				// roughly encompases the whole cloud
	DXVEC2 vCloudHeight;

	DWORD dwNumVerts;
	LPD3DVB pvbPoints;				// generated particles, on write store these particles in a special file

	// run-time only vars
	float fCloudTime, fCloudAlpha;
	D3DXHANDLE hcRad, hcHeight, hcSize, hcTime, hcTexDims, hcAlpha;

	static EventNames evnames;
};

// clouds that surround most of the sky and flash thunder
class LevelThunderClouds : public LevelItem
{
	friend class Level;
	friend class CThunderCloudsItemDlg;

public:

	static const UINT ET_InRange;
	static const UINT ET_NumEvents;

	struct EventNames
	{
		EventNames()
		{
			vecNames.resize(ET_NumEvents);
			vecNames[ET_InRange] = _T("in range");
		}

		vector< sfstring > vecNames;
	};

#ifdef SHADOWFRAME_EDITOR
	bool Write(FILE* f);
#endif

	bool Read(FILE *f, LPD3DDEV pd3dDevice);

	LIContactCode Move(float fTimeElapsed);
	LIContactCode MakeContact(SpaceObject& pship);

	bool DeleteWpnOnCollision()							{ return false; }
	bool DisappearOnDeactivation()								{ return true; }

	UINT GetShipColIndex()								{ return ET_InRange; }

	DWORD						GetNumEvents()			{ return ET_NumEvents; }

protected:
	LevelThunderClouds(DWORD dwNewId);
	~LevelThunderClouds();

	vector< sfstring >&				GetEventNameVec() const			{ return evnames.vecNames; }

	static EventNames evnames;
};

class LevelRain : public LevelItem
{
	friend class Level;
	friend class CRainItemDlg;

public:

	static const UINT ET_InRange;
	static const UINT ET_NumEvents;

	struct EventNames
	{
		EventNames()
		{
			vecNames.resize(ET_NumEvents);
			vecNames[ET_InRange] = _T("in range");
		}

		vector< sfstring > vecNames;
	};

#ifdef SHADOWFRAME_EDITOR
	bool Write(FILE* f);
#endif

	bool Read(FILE *f, LPD3DDEV pd3dDevice);

	LIContactCode Move(float fTimeElapsed);
	LIContactCode MakeContact(SpaceObject& pship);

	bool DeleteWpnOnCollision()							{ return false; }
	bool DisappearOnDeactivation()								{ return true; }

	UINT GetShipColIndex()								{ return ET_InRange; }

	DWORD						GetNumEvents()			{ return ET_NumEvents; }

protected:
	LevelRain(DWORD dwNewId);
	~LevelRain();

	vector< sfstring >&				GetEventNameVec() const			{ return evnames.vecNames; }

	static EventNames evnames;
};

class LevelWind : public LevelItem
{
	friend class Level;
	friend class CWindItemDlg;

public:

	static const UINT ET_InRange;
	static const UINT ET_NumEvents;

	struct EventNames
	{
		EventNames()
		{
			vecNames.resize(ET_NumEvents);
			vecNames[ET_InRange] = _T("in range");
		}

		vector< sfstring > vecNames;
	};

#ifdef SHADOWFRAME_EDITOR
	bool Write(FILE* f);
#endif

	bool Read(FILE *f, LPD3DDEV pd3dDevice);

	LIContactCode Move(float fTimeElapsed);
	LIContactCode MakeContact(SpaceObject& pship);

	bool DeleteWpnOnCollision()							{ return false; }
	bool DisappearOnDeactivation()								{ return true; }

	UINT GetShipColIndex()								{ return ET_InRange; }

	DWORD						GetNumEvents()			{ return ET_NumEvents; }

protected:
	LevelWind(DWORD dwNewId);
	~LevelWind();

	vector< sfstring >&				GetEventNameVec() const			{ return evnames.vecNames; }

	static EventNames evnames;
};

// Other Background objects that are not level items
class SFSkyDome
{
public:
	SFSkyDome();
	~SFSkyDome();

	HRESULT Init(LPD3DDEV pd3dDevice);
	void SetConsts();

	HRESULT LoadVideoRes(LPD3DDEV pd3dDevice);
	void ReleaseVideoRes();

	HRESULT Render(LPD3DDEV pd3dDevice, const DXVEC3& vCamPos);

private:
	LPD3DVB pvb;		// pvbsun just contains one vec: 0,0,0
	LPD3DIB pib;
	DXVEC4 vSkyScale;	// radiusx, radiusy, height, 1
	float fSunSize;

	DWORD dwNumVerts, dwNumFaces;
	LPD3DDECL pdecl;

	float fScaleOverDepth;

	WORD wG, wSunInt, wKray, wKmie, wGroundPlane, wSkyHeight, wiSkyHeight, wInnerRadius, wScaleDepth;	// ids to SFRenderMngr::mapShaderConsts
	D3DXHANDLE hSkyPP, hSunTech;		// techniques
	D3DXHANDLE hKrESun, hKmESun, hKr4PI, hKm4PI, hStartHeight,
				hScaleOverScaleDepth, hDepth, hv3Start, hSkyRadius, hCompDepth, hKrSunWave, hScatterScale;


	ID3DXEffect* peff;			// sky effect
	map<WORD, D3DXHANDLE> mapSkyHandles;		// handles to the effect constants
};
