#ifndef SPACEOBJECT_BASECLASS
#define SPACEOBJECT_BASECLASS

#define DOAI_NONE			0
#define DOAI_HYPERSPACE		1

using namespace std;

class SpaceObject : public SFWorldObj
{
	friend class BaseWeapons;
	friend class Factory;
	friend class SpaceObjectList;			// has to compute bounding boxes for hyperspace objects

public:

	virtual void Destroy();

	virtual HRESULT Init(Info* pNewInfo, DXMAT&, LPD3DDEV pd3dDevice);

	virtual DWORD DoAI();
	virtual void CalcVariables();
	virtual LIContactCode Move(float fTimeElapsed);
	virtual void Explode() = 0;

	virtual HRESULT Render(LPD3DDEV pd3dDevice, RenderPass passtype = RP_Normal) { return S_OK; }

	inline bool			IsInit() const		{ return bInit; }
	inline AABB&		GetVelocityAABB()	{ return abVel; }
	inline Info*		GetInfo() const		{ return info; }
	inline float		GetShields() const	{ return fShields; }
	inline float		GetHull() const		{ return fHull; }

	inline float		GetCurSpeed() const	{ return fCurSpeed; }
	inline float		GetRealSpeed() const{ return fRealSpeed; }
	inline DXVEC3& GetSpeedVec()		{ return vSpeed; }
	inline float		GetRotSpeed()		{ return fRotSpeed; }
	inline DXVEC3& GetRotVec()			{ return vRotVel; }
	inline bool			IsNormalState()		{ return bNormalState; }
	inline bool			IsInLevel()			{ return bIsInLevel; }
	inline ClassData*	GetData()			{ return (ClassData*)pObj; }

	void				SetInLevel(bool bNewIsInLevel)			{ bIsInLevel = bNewIsInLevel; }

	WORD				GetClass() const;
	SideType			GetSide() const;
	bool				IsImportant() const;
	TCHAR*				GetName() const;	

	virtual float		GetStupidity() const = 0;
	virtual float		GetMass();

	void				SetCurSpeed(float fNewSpeed);
	void				AddToLinVel(DXVEC3& vAddVel);
	void				AddToRotVel(DXVEC3& vAddVel);

	void				SetLevelSeparation(DXVEC3& vNewSep)	{ vLevelSeparation = vNewSep; }

	virtual inline DWORD	GetGroupId()		{ return 0; }

	virtual inline float	GetShieldDef() = 0;
	virtual inline float	GetHullDef() = 0;

	virtual WEAPONVALUES*	GetWpnValues(UINT uWpnType) = 0;

	virtual inline bool		InSight(SpaceObject* pTarget)	{ return false; }
	virtual inline bool		IsAttacking(DWORD dwId)			{ return false; }
	virtual inline bool		IsLocking()						{ return false; }
	virtual inline bool		IsNearDeath()					{ return false; }
	virtual inline bool		IsLeader()						{ return false; }
	virtual inline bool		IsShooting()					{ return false; }
	virtual inline bool		IsStealthMode()					{ return false; }
	virtual inline float	GetLockingDifficulty()			{ return 1.0f; }

	virtual void			NotifyGroupMemberDestroyed()	{}
	virtual void			HeadlessSnakeMode()				{ bNormalState = false; }

	// in COLLISIONINFO, this object's info is in pObj1 and dwExtra1
	// called whenever weapon hits the object
	virtual void			AddShipExp(float fKillExp) {}					// so ship can upgrade levels
	virtual void			AddWeaponExp(int iType, float fKillExp) {}		// for weapons
	virtual void			WpnHitEnergy(float fEnergy, DWORD dwAttackerId, UINT iWpnType, COLLISIONINFO*);
	virtual void			IncHitCount(DWORD dwHitShipId)	{}

	virtual			SpaceObject* FindNearestAttacker(float* fInfluence)			{ return NULL; }

	virtual float	GetShipInfluence(SpaceObject& pAttackingObj)				{ return 0.5f; }

	DXVEC3 vNewVel;
	DXVEC3 vAccel;					// rotational acceleration
	DXVEC3 vRightAccel;

protected:

	SpaceObject(DWORD dwNewId, DWORD dwNewType, DWORD dwNewDB);
	virtual ~SpaceObject();

	virtual void SetTrans(DXMAT& mNew);

	Info* info;

	DXVEC3 vRotVel;				// the rotational velocity (normalized)
	DXVEC3 vSpeed;					// the actual speed counting the linear velocity (normalized)
	float fRealSpeed;					// the real speed counting fCurSpeed and vLinAccel
	float fCurSpeed;					// the speed of the object from its own engines and internal forces
	float fRotSpeed;					// the speed of the rotational velocity in radians per second
	float fPrevSpeed;					// the previous speed
	float fRoll;						// amount to roll by
	float fiRotInertia;					// 1 / Rotational inertia
	
	DXVEC3 vLevelSeparation;		// separation from the level

	AABB abVel;					// the AABB that surrounds the whole ship and its future position
	DWORD dwLastAttackerId;		// the id of the ships that last attacked this ship

	bool bInit;					// is the object active in the scene?
	bool bNormalState;			// true if in normal state,
								// false if has leader and he was just killed

	bool bIsInLevel;			// some objects won't be attached to the sections and can move around
								// The problem occurs is when they are really far away from the player, so far
								// that not even the collision data for the sections is loaded. So if this flag is
								// false, don't even move this object. This is updated in Mission::Move and should
								// be checked explicitly by SFList and Mission

	float fShields;						// percentage of shields remaining 0-1
	float fHull;						// percentage of hull remaining 0-1

	float ifShieldDef, ifHullDef;		// 1 / fShieldDef and 1 / fHullDef
};

// functions //

// returns 0 if the weapon won't hit the space object at all
// otherwise returns a range from 0 to 1 indicating how good
// of an aim the current ship has
float	GetWpnAimValue(UINT uWpnType, SpaceObject& p, SpaceObject& enemy);
bool	ShipIntersect(float fWpnSpeed, DXVEC3& vPos, float fCurSpeed, SpaceObject& enemy, DXVEC3& vFinalPos);

#endif