float4 c_SunColor;
float3 c_SunPos;

float4x4 c_ViewProj;
float4x4 c_WVP;

int c_Samples = 2;
float fiSamples = 1.0 / 2.0;

// Atmosphere scattering code mostly from Sean O'Neil (GPU Gems 2, pg. 253-268)

// The scale depth (the altitude at which the average atmospheric density is found)
const float c_ScaleDepth = 0.25;
float3 c_InvWavelength;			// 1 / pow(wavelength, 4) for the red, green, and blue channels
float4 c_GroundPlane;			// dot product with this plane + w = height from ground
float4 c_CameraPos;
float4 c_SunDir;
float2 c_InnerRadius;		// The inner (planetary) radius (radius, 0)
float c_SkyHeight;
float4 c_iSkyHeight;				// 0.5, 0.25, 0.5 / c_SkyHeight, 1 / c_SkyHeight

// atmospheric consts
float c_Kray, c_Kmie, c_SunIntensity;
float c_mieG = 0.75f;

#include "Common.fx"

struct PSINPUT
{
	float4 pos : POSITION0;
	float3 c0 : COLOR0;
	float3 c1 : COLOR1;
	half3 vray : TEXCOORD0;
};

float fKrESun; // c_Kray * c_SunIntensity;
float fKmESun; // c_Kmie * c_SunIntensity;
float fKr4PI; // c_Kray * 4.0f * 3.14259268f;
float fKm4PI; // c_Kmie * 4.0f * 3.14159268f;
float fStartHeight; // dot(c_GroundPlane, c_CameraPos);
//float4 fSkyOffset;
float4 fSkyRadius;			// amount to scale sky dome (radius, sky height, offset, 0)
float fScaleOverScaleDepth; // c_iSkyHeight / c_ScaleDepth;
float fDepth; // exp( -dot(c_GroundPlane, c_CameraPos) * fScaleOverScaleDepth);
float3 v3Start; // c_InnerRadius.yxy * c_GroundPlane.xyz;
float c_ElapsedTime;

float2 fCompDepth;		// fScaleOverScaleDepth * c_InnerRadius, -fScaleOverScaleDepth
float3 fKrSunWave;		// c_InvWavelength * fKrESun
float3 fScatterScale;		// c_InvWavelength * fKr4PI + fKm4PI

PSINPUT SkyVS(float4 pos : POSITION0)
{
	PSINPUT o;

	float4 vray;
	vray.xyz = pos.xyz * fSkyRadius.xyx + fSkyRadius.wzw;
	vray.w = 1;
	o.pos = mul(vray, c_WVP);

	vray.w = dot(vray.xyz, vray.xyz);
	vray *= rsqrt(vray.w);

	//float fc = cos(0.4*c_ElapsedTime), fs = (sin(0.4*c_ElapsedTime));
	//float3 c_SunDir = float3(fc,fs, 0);

	// Calculate the ray's starting position, then calculate its scattering offset
	//float fStartAngle = dot(vray.xyz, c_GroundPlane.xyz);
	float fStartOffset = fDepth*getScatterScale(vray.y);

	// .x - sample length, .y - 0.5 * sample length, .z is scaled length
	float3 fSampleLength = vray.www * c_iSkyHeight.xyz;
	float3 v3SamplePoint = v3Start + vray.xyz * fSampleLength.y;
	//float fHeight = fStartHeight;

	// Now loop through the sample rays
	float3 v3FrontColor = 0;
    for(int i=0; i<2; i++) {
		float4 vdir;
		vdir.xyz = v3SamplePoint.xyz;
		vdir.w = dot(v3SamplePoint.xyz, v3SamplePoint.xyz);
		vdir *= rsqrt(vdir.w);
		float fd = exp2(fCompDepth.x + fCompDepth.y * vdir.w);
		float fLightAngle = dot(vdir.xyz, c_SunDir.xyz);
		float fCameraAngle = dot(vdir.xyz, vray.xyz);
		float fScatter = fStartOffset + fd*(getScatterScale(fLightAngle) - getScatterScale(fCameraAngle));
		float3 v3Attenuate = exp(-fScatter * fScatterScale);
		v3FrontColor += v3Attenuate * fd;
		v3SamplePoint.xyz += vray.xyz * fSampleLength.x;
	}

	// Finally, scale the Mie and Rayleigh colors and set up the varying variables for the pixel shader
	v3FrontColor *= fSampleLength.z;
	o.c0 = v3FrontColor * fKrSunWave;//(c_InvWavelength * fKrESun);
	o.c1 = v3FrontColor * fKmESun;
	o.vray = vray;

	return o;
}

struct PSOUT
{
       float4 col0 : COLOR0;
       float4 col1 : COLOR1;
};

PSOUT SkyPS( PSINPUT i ) {
	PSOUT o;
	float fCos = dot(normalize(i.vray.xyz), c_SunDir.xyz);
	float fCos2 = fCos * fCos;
	float fCos4 = fCos2*fCos2;
	o.col0.xyz = i.c0 + getMiePhase(fCos, fCos2) * i.c1;
	o.col0.w = 1;
	o.col1 = 0.2*fCos4*fCos4+saturate((o.col0 - 0.9)/4 ) + 0.0045;

	return o;
}

technique tech0
{
    pass p0
    {
		ZWriteEnable = False;
        VertexShader = compile vs_2_0 SkyVS();
		PixelShader = compile ps_2_0 SkyPS();
    }
}
